/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */
import Foundation

/**
  Data manager allows BlueConic to write and read files from the document file system.
 */
class BlueConicDataManager {
    init() {
    }

    /**
     Write data to file, stores the data based on the file name.
     - parameter data: Generics Codable object, the object much extend Codable.
     - parameter fileName: the location on where to store the data to, see getFileLocation on how to retrieve the fileName.
     */
    class func writeToFile<T: Codable>(_ data: T, fileName: String) {
        do {
            let encoder = JSONEncoder()
            let encodedData = try encoder.encode(data)

            if let data = String(data: encodedData, encoding: .utf8) {
                writeStringToFile(data: data, fileName: fileName)
            }
        } catch {
            BCLog("Unable to encoded data! \(error)")
        }
    }

    /**
     Write string to file, stores the data based on the file name.
     - parameter data: The content that is stored in the file.
     - parameter fileName: the location on where to store the data to, see getFileLocation on how to retrieve the fileName.
     */
    class func writeStringToFile(data: String, fileName: String) {
        do {
            if let file: String = getFileLocation(fileName) {
                try data.write(toFile: file, atomically: true, encoding: .utf8)
            }
        } catch {
            BCLog("Unable to write to file! \(error)")
        }
    }

    /**
     Load data from file.
     - parameter fileName: the location on where to store the data to, see getFileLocation on how to retrieve the fileName.
     - parameter type: The type needs to be the object type to transform the Codable object, to the correct object reference.
     - returns: stongly-typed reference of the data stored at the file system.
     */
    class func loadFromFile<T: Codable>(fileName: String, type: T.Type) -> T? {
        do {
            if let content = loadStringFromFile(fileName: fileName),
                let data = content.data(using: .utf8) {
                    let decoder = JSONDecoder()
                    return try decoder.decode(type, from: data)
            }
        } catch {
            BCLog("Unable to decode data! \(error)")
        }
        return nil
    }

    /**
     Load string from file.
     - parameter fileName: the location on where to store the data to, see getFileLocation on how to retrieve the fileName.
     - returns: the data of which is stored at the file system.
     */
    class func loadStringFromFile(fileName: String) -> String? {
        guard let file: String = getFileLocation(fileName),
            FileManager.default.fileExists(atPath: file) else {
                return nil
        }

         do {
            return try String(contentsOfFile: file, encoding: .utf8)
         } catch {
            BCLog("Unable to load the file! \(error)")
            do {
                try FileManager.default.removeItem(atPath: file)
            } catch {
                BCLog("Unable to remove! \(error)")
            }
        }
        return nil
    }

    /*
     Return the file directory of a filename as a String.

     - parameter fileName: The name of a file, which is in the BlueConic Directory.
     - returns: A string with the file location.
     */
    class func getFileLocation(_ fileName: String) -> String? {
        if let bcDirectory = getFolderLocation(),
            let url = URL(string: bcDirectory) {
                return url.appendingPathComponent(fileName).path
        }
        return nil
    }

    /*
     Return the folder directory

     - returns: A string with the folder location.
     */
    class func getFolderLocation() -> String? {
        // Get the documents directory
        do {
            let url = try FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: true)
            let directory = url.appendingPathComponent(Constants.Files.BLUECONIC_DIR).path
            try FileManager.default.createDirectory(atPath: directory, withIntermediateDirectories: true, attributes: nil)
            return directory
        } catch {
            BCLog("Unable to get folder location")
        }

        return nil
    }

    /**
     Clean up the old commitlogs and cache files.
     */
    class func cleanUpFiles() {
        do {
            guard let blueConicFolder = getFolderLocation() else {
                BCLog("Unable to clean up, because the blueconic folder is not available")
                return
            }

            let dictionaryContents = try FileManager.default.contentsOfDirectory(atPath: blueConicFolder)
            for fileName in dictionaryContents {
                if fileName.range(of: Constants.SWIFT_VERSION) == nil && fileName != Constants.Files.ID {
                    // Only remove the file if it is a BlueConic file
                    if Constants.Files.BC_FILES.filter({(item: String) -> Bool in
                        return fileName.range(of: item) != nil
                    }).count > 0 {
                        if let path = self.getFileLocation(fileName) {
                            try FileManager.default.removeItem(atPath: path)
                        }
                    }
                }
            }
        } catch {
            BCLog("Unable to clean up files")
        }
    }

    private class func BCLog(_ message: String, flag: String = Constants.Flag.ERROR) {
        BlueConicConfiguration.log(message, tag: Constants.Debug.DATA_MANAGER, flag: flag)
    }
}
